from .FormField import FormField, FormFieldType, FormKeyboardType
from .Actionable import Actionable, ActionableType, URI
from .CarouselItem import CarouselItem
from .SilentAction import SilentAction

__all__=[
    'FormField',
    'Actionable',
    'SilentAction',
    'CarouselItem',
    'ActionableType',
    'URI',
    'FormFieldType',
    'FormKeyboardType'
]



from typing import List
from .Base import BaseElement
from .Elements.Actionable import Actionable

class Button(BaseElement):
    """
    Create Button HSL Elements

    Attributes
    ----------
    actionables : List[Actionable]
        list of actionables that will be added to the button

    """
    def __init__(self,text):
        super().__init__(text,'BUTTON')
        self.actionables: List[Actionable] = []

    def to_hsl(self):
        """
        Generate HSL dict
        """
        hsl = super().to_hsl()
        data = {
            'items': [actionable.to_hsl() for actionable in self.actionables]
        }
        hsl['data'] = data

        return hsl



from .Actionable import Actionable
from typing import List

class CarouselItem(object):
    """
    Create Carousel Items for Carousel

    Attributes
    ----------
    title : str
        Title of specific carousel item.
        
    subtitle : str
        Subtitle of specific carousel item.
        
    description : str
        More verbose description of item that is shown below the subtitle

    actionables : List[Actionable]
        Actionables to be performed when the user taps on the item

    meta : str
        meta value for the corresponding carousal item
    """


    def __init__(self,title: str, subtitle: str):
        self.title: str = title
        self.subtitle: str = subtitle
        self.description: str = ''
        self.thumbnail: str = ''
        self.actionables: List[Actionable] = []
        self.meta: str = ''

    def to_hsl(self) -> dict:
        """
        Generate HSL dict
        """
        return {
            'title': self.title,
            'sub_title': self.subtitle,
            'description': self.description,
            'actionables': [actionable.to_hsl() for actionable in self.actionables],
            'meta': self.meta,
            'thumbnail': {
                'image': self.thumbnail
                }
        }



class SilentAction(object):
    """
    Create Silent Actions for Silent Message HSl

    Attributes
    ----------
    type : str
        Type of silent action.
        
    via_name : str
        via_name of the corresponding business.
        
        
    action_id : str
        id of the silent action.
        
    data : dict
        options that will be sent as payload
    """

    def __init__(self, action_type: str, id: int, via_name: str):
        self.type: str = action_type
        self.via_name: str = via_name
        self.action_id: int = id
        self.data: dict = {}

    def to_hsl(self) -> dict:
        """
        Generate HSL dict
        """
        self.data['Id'] = self.action_id
        self.data['via_name'] = self.via_name
        return {
            "type": self.type,
            "data": self.data
        }



from .Base import BaseElement
from enum import unique,Enum

@unique
class SystemEvents(Enum):
    PINNED = 'chat_pinned'
    COMPLETE = 'chat_complete'

class System(BaseElement):
    """
    create system messages hsl

    Attributes
    ----------
    event_name : SystemEvents
        the type of the System Event to be sent

    payload : dict
        additonal data sent as payload
    """
    def __init__(self,text :str,event_name: SystemEvents):
        super().__init__(text,'SYSTEM')
        self.event = event_name
        self.payload = {}

    def to_hsl(self):
        hsl = super().to_hsl()
        data = {
            'event_name': self.event.value,
            'payload': self.payload
        }
        hsl['data'] = data

        return hsl



class BaseElement(object):
    """
    Base Class containing common properties for all HSL Messages

    Attributes
    ----------
    text : str
        text for the message to be displayed to the user

    type : str
        type of the HSL Message

    voice_text : str
        this text will be spoken out if tts is enabled
    """

    def __init__(self, text: str, hsl_type: str):
        self.text: str = text
        self.type: str = hsl_type
        self.voice_text: str = ""

    def to_hsl(self):
        """
        Generate HSL dict
        """
        return {
            'text': self.text,
            'type': self.type,
            'voice_text': self.voice_text
        }



from .Base import BaseElement
from typing import List

from .Elements import SilentAction

class Silent(BaseElement):
    """
    Create Silent Message HSL

    Attributes
    ----------
    actions : List[SilentAction]
        list of silent acitons to be taken
    """
    def __init__(self,text: str):
        super().__init__(text,'SILENT')
        self.actions: List[SilentAction] = []

    def to_hsl(self):
        """
        Generate HSL dict
        """
        hsl = super().to_hsl()
        data = {
            'silent_actions': [action.to_hsl() for action in self.actions]
        }
        hsl['data'] = data

        return hsl



from typing import List
from .Base import BaseElement
from .Elements.Actionable import Actionable

class Text(BaseElement):
    """
    Create simple text message HSL
    """
    def __init__(self,text):
        super().__init__(text,'TEXT')
        self.quick_replies: List[Actionable] = []
    def to_hsl(self):
        """
        Generate HSL message
        """
        hsl = super().to_hsl()
        data = {
            'quick_replies': [qr.to_hsl() for qr in self.quick_replies]
        }
        hsl['data'] = data

        return hsl



from typing import List

from .Elements.FormField import FormField


class Form(object):
    """
    Create HSL Form message.

    Attributes
    ----------
    title : str
        the title of the form.

    subtitle : str
        Sub title for the form for showing more info

    fields : List[FormField]
        list of formfields that will be added to the form

    """
    def __init__(self,text: str, subtitle: str):
        self.title: str = text
        self.type: str = 'FORM'
        self.subtitle: str = subtitle
        self.fields: List[FormField] = []

    def to_hsl(self):
        """
        Generate HSL dict
        """
        hsl = {
            'title': self.title,
            'type': self.type,
            'subtitle': self.subtitle
        }
        data = {
            'fields': [field.to_hsl() for field in self.fields]
        }
        hsl['data'] = data

        return hsl



from enum import Enum,unique
from typing import List

from .Base import BaseElement
from .Elements.CarouselItem import CarouselItem

@unique
class CarouselWidth(Enum):
    THIN = 'THIN'
    MEDIUM = 'MEDIUM'
    FAT = 'FAT'
    BIG = 'BIG'


class Carousel(BaseElement):
    """
    Create Carousel HSL message

    Attributes
    ----------
    aspect_ratio : float
        used to determine the height of the carousel

    width : `CarouselWidth`
        The width of each carousel Item

    items : List[CarouselItem]
        list of carousel items for the carousel
    """
    def __init__(self,text):
        super().__init__(text,'CAROUSEL')
        self.aspect_ratio: float = 1.0
        self.width: CarouselWidth = CarouselWidth.THIN
        self.items: List[CarouselItem] = []

    def to_hsl(self):
        """
        Generate HSL dict
        """
        hsl = super().to_hsl()
        data = {
            'image_aspect_ratio': self.aspect_ratio,
            'width': self.width.value,
            'items': [item.to_hsl() for item in self.items]
        }
        hsl['data'] = data

        return hsl



from .Button import Button
from .Carousel import Carousel, CarouselWidth
from .Form import Form
from .NativeAction import NativeAction
from .Silent import Silent
from .Text import Text
from .System import System, SystemEvents
from .Base import BaseElement


__all__= [
    'Button',
    'CarouselWidth',
    'Carousel',
    'Form',
    'NativeAction',
    'Silent',
    'Text',
    'System',
    'SystemEvents',
    'BaseElement'
]



from enum import unique,Enum

@unique
class URI(Enum):
    """
    URI Types for showing which screen to open in case of app_action
    """
    NONE = ''
    SEND_LOCATION = 'SEND_LOCATION'
    CAROUSEL_DETAIL = 'CAROUSEL_DETAIL'
    GALLERY_PICKER = 'GALLERY_PICKER'
    IMAGE_UPLOAD = 'IMAGE_UPLOAD'
    DOCUMENT_PICKER = 'DOCUMENT_PICKER'
    LAUNCH_CHANNEL = 'LAUNCH_CHANNEL'
    LINK = 'LINK'
    SELF_SERVE_WEB = 'SELF_SERVE_WEB'


@unique
class ActionableType(Enum):
    """
    Actionable Types for specifying what action should be taken 
    """
    APP_ACTION = 'APP_ACTION'
    MESSAGE_BAR = 'MESSAGE_BAR'
    TEXT_ONLY = 'TEXT_ONLY'
    FORM_SHOW = 'FORM_SHOW'
    SHARE_RECEIPT = 'SHARE_RECEIPT'
    APP_FEEDBACK = 'APP_FEEDBACK'
    SHARE = 'SHARE'
 
class Actionable(object):
    """
    Create Actionables to be used in Buttons or Text messages

    Attributes
    ----------
    text : str
        Text for the Actionable that will be displayed on the CTA (button, quick reply,etc).
        
    type : ActionableType
        The type of `ActionableType`.
        
    type : URI
        The type of `URI`.
        
    location_required : bool
        True if location is required for using the Actionable.

    is_default : bool
        if True, on click of the element, this actionable will get triggered.

    payload : dict 
        optional payload containing metadata that might be needed by the Actionable
    """

    def __init__(self, text: str, type: ActionableType, uri: URI):
        self.text: str = text
        self.type: ActionableType = type
        self.uri: URI = uri
        self.location_required: bool = False
        self.is_default: bool = False
        self.payload: dict = {}

    def to_hsl(self) -> dict:
        """
        Generate HSL dict
        """
        return {
            'actionable_text': self.text,
            'type': self.type.value,
            'uri': self.uri.value,
            'is_default': 1 if self.is_default else 0,
            'location_required': self.location_required,
            'payload': self.payload
        }



from .Base import BaseElement


class NativeAction(BaseElement):
    """
    Create Native Action HSL Message.
    Native Actions are used to communicate with the parent app in which SDK is integrated

    Attributes
    ----------
    method : str
        the method name for the native action 
    """
    def __init__(self,text: str, method: str):
        super().__init__(text,'NATIVE_ACTION')
        self.method = method

    def to_hsl(self):
        """
        Generate HSL dict
        """
        hsl = super().to_hsl()
        data = {
            'method': self.method
        }
        hsl['data'] = data

        return hsl



from enum import Enum,unique
from typing import List

@unique
class FormFieldType(Enum):
    TEXT = 'text'
    PICKER = 'picker'
    TIME = 'time'
    DATE = 'date'
    CONTACT_PICKER = 'contactpicker'
    START_DATE = 'startdate'
    END_DATE = 'enddate'
    SEARCH = 'search'
    SEARCH_EDITABLE = 'searcheditable'
    SAVED_ADDRESS = 'savedaddress'
    DOB = 'dob'
    MULTI_DAY_PICKER = 'multidaypicker'
    MULTI_SELECT_PICKER = 'multiselectpicker'

@unique
class FormKeyboardType(Enum):
    NORMAL = 'normal'
    NUMBER = 'number'
    EMAIL = 'email'


class FormField(object):
    """
     Create FormField objects that can be added in the Form Object

    Attributes
    ----------
    key : str
        When the form is submitted, apps show the value for the form field against this key.
        This key will then be used for entity detection in bot builder

    type : FormFieldType
        The type of the formfield

    keyboard_type : FormKeyboardType
        The keyboard type used to fill the corresponding form

    order : int
        The order of the form

    icon : str 
        Icon name for the field.

    hint : str
        Input placeholder for the field

    options : str
        In case of FormField type picker or multiselectpicker
        These will be used as the options to pick from.

    search_source : str
        API URL to get the search results from

    search_placeholder : str
        placeholder when no text is entered for search type fields

    autofill : str 
        Auto fill field based on user profile or default value.

    auto_fill_source : str
        Source for the autofilling the field
    """
    def __init__(self, key: str, form_type: FormFieldType, order: int, hint: str, icon: str):
        self.key: str = key
        self.type: FormFieldType = form_type
        self.keyboard_type: FormKeyboardType = FormKeyboardType.NORMAL
        self.order: int = order
        self.icon: str = icon
        self.hint: str = hint
        self.options: List[str] = []
        self.search_source: str = ''
        self.search_placeholder: str = ''
        self.autofill: str = ''
        self.autofill_source: str = ''

    def to_hsl(self) -> dict:
        """
        returns an HSL dict for the form field
        """
        return {
            'key': self.key,
            'type': self.type.value,
            'keyboard_type': self.keyboard_type.value,
            'order': self.order,
            'icon': self.icon,
            'hint': self.hint,
            'options': self.options,
            'search_source': self.search_source,
            'search_placeholder': self.search_placeholder,
            'autofill': self.autofill,
            'autofill_source': self.autofill_source,
            #TODO: add regex fields
        }



from setuptools import setup,find_packages


with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='hsl_builder',
    version='0.0.1-alpha',
    description='HSL Builder for creating hsl elements',
    license='MIT',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=('tests',)),
    zip_safe=False,
    url="https://github.com/hellohaptik/python-hsl")
